@@ -97,6 +97,32 @@ module Spec
       end
     end
 
+    RSpec::Matchers.define :take_less_than do |seconds|
+      match do |actual|
+        start_time = Time.now
+
+        actual.call
+
+        actual_time = (Time.now - start_time).to_f
+
+        acceptable = actual_time < seconds
+
+        @errors = ["took #{actual_time} seconds"] unless acceptable
+
+        acceptable
+      end
+
+      failure_message do
+        super() + " but:\n" + @errors.map {|e| indent(e) }.join("\n")
+      end
+
+      failure_message_when_negated do
+        super() + " but:\n" + @errors.map {|e| indent(e) }.join("\n")
+      end
+
+      supports_block_expectations
+    end
+
     define_compound_matcher :read_as, [exist] do |file_contents|
       diffable
 
@@ -150,7 +176,7 @@ module Spec
           end
           if exitstatus == 65
             actual_platform = out.split("\n").last
-            next "#{name} was expected to be of platform #{platform} but was #{actual_platform}"
+            next "#{name} was expected to be of platform #{platform || "ruby"} but was #{actual_platform || "ruby"}"
           end
           if exitstatus == 66
             actual_source = out.split("\n").last
@@ -178,7 +204,7 @@ module Spec
 
             begin
               require '#{name}'
-              name_constant = '#{Spec::Builders.constantize(name)}'
+              name_constant = #{Spec::Builders.constantize(name)}
               if #{version.nil?} || name_constant == '#{version}'
                 exit 64
               else
@@ -208,10 +234,6 @@ module Spec
     RSpec::Matchers.define_negated_matcher :not_include_gems, :include_gems
     RSpec::Matchers.alias_matcher :include_gem, :include_gems
 
-    def have_lockfile(expected)
-      read_as(strip_whitespace(expected))
-    end
-
     def plugin_should_be_installed(*names)
       names.each do |name|
         expect(Bundler::Plugin).to be_installed(name)
@@ -225,13 +247,5 @@ module Spec
         expect(Bundler::Plugin).not_to be_installed(name)
       end
     end
-
-    def lockfile_should_be(expected)
-      expect(bundled_app_lock).to have_lockfile(expected)
-    end
-
-    def gemfile_should_be(expected)
-      expect(bundled_app_gemfile).to read_as(strip_whitespace(expected))
-    end
   end
 end
