@@ -34,12 +34,6 @@ describe "RUBY_PLATFORM" do
   it "is a String" do
     RUBY_PLATFORM.should be_kind_of(String)
   end
-
-  platform_is :darwin do
-    it 'ends with the build time kernel major version on darwin' do
-      RUBY_PLATFORM.should =~ /-darwin\d+$/
-    end
-  end
 end
 
 describe "RUBY_RELEASE_DATE" do
@@ -49,15 +43,7 @@ describe "RUBY_RELEASE_DATE" do
 end
 
 describe "RUBY_REVISION" do
-  ruby_version_is ""..."2.7" do
-    it "is an Integer" do
-      RUBY_REVISION.should be_kind_of(Integer)
-    end
-  end
-
-  ruby_version_is "2.7" do
-    it "is a String" do
-      RUBY_REVISION.should be_kind_of(String)
-    end
+  it "is a String" do
+    RUBY_REVISION.should be_kind_of(String)
   end
 end
