@@ -40,7 +40,7 @@ class Gem::FakeFetcher
   end
 
   def find_data(path)
-    return Gem.read_binary path.path if URI === path && "file" == path.scheme
+    return Gem.read_binary path.path if URI === path && path.scheme == "file"
 
     if URI === path && "URI::#{path.scheme.upcase}" != path.class.name
       raise ArgumentError,
@@ -55,7 +55,7 @@ class Gem::FakeFetcher
       raise Gem::RemoteFetcher::FetchError.new("no data for #{path}", path)
     end
 
-    if @data[path].kind_of?(Array)
+    if @data[path].is_a?(Array)
       @data[path].shift
     else
       @data[path]
@@ -65,7 +65,7 @@ class Gem::FakeFetcher
   def create_response(uri)
     data = find_data(uri)
     response = data.respond_to?(:call) ? data.call : data
-    raise TypeError, "#{response.class} is not a type of Net::HTTPResponse" unless response.kind_of?(Net::HTTPResponse)
+    raise TypeError, "#{response.class} is not a type of Gem::Net::HTTPResponse" unless response.is_a?(Gem::Net::HTTPResponse)
 
     response
   end
@@ -120,7 +120,7 @@ class Gem::FakeFetcher
     path = path.to_s
     @paths << path
 
-    raise ArgumentError, "need full URI" unless path =~ %r{^http://}
+    raise ArgumentError, "need full URI" unless %r{^http://}.match?(path)
 
     unless @data.key? path
       raise Gem::RemoteFetcher::FetchError.new("no data for #{path}", path)
@@ -141,7 +141,7 @@ class Gem::FakeFetcher
 
     path = File.join path, name
 
-    if source_uri =~ /^http/
+    if /^http/.match?(source_uri)
       File.open(path, "wb") do |f|
         f.write fetch_path(File.join(source_uri, "gems", name))
       end
@@ -164,7 +164,7 @@ class Gem::FakeFetcher
 end
 
 ##
-# The HTTPResponseFactory allows easy creation of Net::HTTPResponse instances in RubyGems tests:
+# The HTTPResponseFactory allows easy creation of Gem::Net::HTTPResponse instances in RubyGems tests:
 #
 # Example:
 #
@@ -178,7 +178,7 @@ end
 
 class Gem::HTTPResponseFactory
   def self.create(body:, code:, msg:, headers: {})
-    response = Net::HTTPResponse.send(:response_class, code.to_s).new("1.0", code.to_s, msg)
+    response = Gem::Net::HTTPResponse.send(:response_class, code.to_s).new("1.0", code.to_s, msg)
     response.instance_variable_set(:@body, body)
     response.instance_variable_set(:@read, true)
     headers.each {|name, value| response[name] = value }
@@ -201,23 +201,23 @@ end
 
 class Gem::MockBrowser
   def self.options(uri)
-    options = Net::HTTP::Options.new(uri)
-    Net::HTTP.start(uri.hostname, uri.port) do |http|
+    options = Gem::Net::HTTP::Options.new(uri)
+    Gem::Net::HTTP.start(uri.hostname, uri.port) do |http|
       http.request(options)
     end
   end
 
   def self.get(uri)
-    get = Net::HTTP::Get.new(uri)
-    Net::HTTP.start(uri.hostname, uri.port) do |http|
+    get = Gem::Net::HTTP::Get.new(uri)
+    Gem::Net::HTTP.start(uri.hostname, uri.port) do |http|
       http.request(get)
     end
   end
 
   def self.post(uri, content_type: "application/x-www-form-urlencoded")
     headers = { "content-type" => content_type } if content_type
-    post = Net::HTTP::Post.new(uri, headers)
-    Net::HTTP.start(uri.hostname, uri.port) do |http|
+    post = Gem::Net::HTTP::Post.new(uri, headers)
+    Gem::Net::HTTP.start(uri.hostname, uri.port) do |http|
       http.request(post)
     end
   end
@@ -366,7 +366,8 @@ class Gem::TestCase::SpecFetcherSetup
     Gem::Specification.reset
 
     begin
-      gem_repo, @test.gem_repo = @test.gem_repo, @repository
+      gem_repo = @test.gem_repo
+      @test.gem_repo = @repository
       @test.uri = URI @repository
 
       @test.util_setup_spec_fetcher(*@downloaded)
