@@ -4,7 +4,7 @@
 
 #include "id_table.h"
 #include "yjit.h"
-#include "mjit.h"
+#include "rjit.h"
 
 #define METHOD_DEBUG 0
 
@@ -19,7 +19,6 @@ static inline rb_method_entry_t *lookup_method_table(VALUE klass, ID id);
 #define singleton_removed   idSingleton_method_removed
 #define undefined           idMethod_undefined
 #define singleton_undefined idSingleton_method_undefined
-#define attached            id__attached__
 
 #define ruby_running (GET_VM()->running)
 /* int ruby_running = 0; */
@@ -125,7 +124,7 @@ vm_cme_invalidate(rb_callable_method_entry_t *cme)
     RB_DEBUG_COUNTER_INC(cc_cme_invalidate);
 
     rb_yjit_cme_invalidate(cme);
-    rb_mjit_cme_invalidate(cme);
+    rb_rjit_cme_invalidate(cme);
 }
 
 static int
@@ -151,7 +150,7 @@ rb_clear_constant_cache_for_id(ID id)
     }
 
     rb_yjit_constant_state_changed(id);
-    rb_mjit_constant_state_changed(id);
+    rb_rjit_constant_state_changed(id);
 }
 
 static void
@@ -190,7 +189,7 @@ clear_method_cache_by_id_in_class(VALUE klass, ID mid)
         if (cc_tbl && rb_id_table_lookup(cc_tbl, mid, &ccs_data)) {
             struct rb_class_cc_entries *ccs = (struct rb_class_cc_entries *)ccs_data;
             rb_yjit_cme_invalidate((rb_callable_method_entry_t *)ccs->cme);
-            rb_mjit_cme_invalidate((rb_callable_method_entry_t *)ccs->cme);
+            rb_rjit_cme_invalidate((rb_callable_method_entry_t *)ccs->cme);
             if (NIL_P(ccs->cme->owner)) invalidate_negative_cache(mid);
             rb_vm_ccs_free(ccs);
             rb_id_table_delete(cc_tbl, mid);
@@ -201,9 +200,11 @@ clear_method_cache_by_id_in_class(VALUE klass, ID mid)
         struct rb_id_table *cm_tbl;
         if ((cm_tbl = RCLASS_CALLABLE_M_TBL(klass)) != NULL) {
             VALUE cme;
-            if (rb_yjit_enabled_p() && rb_id_table_lookup(cm_tbl, mid, &cme)) {
+            if (rb_yjit_enabled_p && rb_id_table_lookup(cm_tbl, mid, &cme)) {
                 rb_yjit_cme_invalidate((rb_callable_method_entry_t *)cme);
-                rb_mjit_cme_invalidate((rb_callable_method_entry_t *)cme);
+            }
+            if (rb_rjit_enabled && rb_id_table_lookup(cm_tbl, mid, &cme)) {
+                rb_rjit_cme_invalidate((rb_callable_method_entry_t *)cme);
             }
             rb_id_table_delete(cm_tbl, mid);
             RB_DEBUG_COUNTER_INC(cc_invalidate_leaf_callable);
@@ -239,6 +240,13 @@ clear_method_cache_by_id_in_class(VALUE klass, ID mid)
                 vm_cme_invalidate((rb_callable_method_entry_t *)cme);
                 RB_DEBUG_COUNTER_INC(cc_invalidate_tree_cme);
 
+                // In case of refinement ME, also invalidate the wrapped ME that
+                // could be cached at some callsite and is unreachable from any
+                // RCLASS_CC_TBL.
+                if (cme->def->type == VM_METHOD_TYPE_REFINED && cme->def->body.refined.orig_me) {
+                    vm_cme_invalidate((rb_callable_method_entry_t *)cme->def->body.refined.orig_me);
+                }
+
                 if (cme->def->iseq_overload) {
                     rb_callable_method_entry_t *monly_cme = (rb_callable_method_entry_t *)lookup_overloaded_cme(cme);
                     if (monly_cme) {
@@ -305,21 +313,22 @@ rb_clear_method_cache(VALUE klass_or_module, ID mid)
 void rb_cc_table_free(VALUE klass);
 
 static int
-invalidate_all_cc(void *vstart, void *vend, size_t stride, void *data)
+invalidate_all_refinement_cc(void *vstart, void *vend, size_t stride, void *data)
 {
     VALUE v = (VALUE)vstart;
     for (; v != (VALUE)vend; v += stride) {
         void *ptr = asan_poisoned_object_p(v);
         asan_unpoison_object(v, false);
+
         if (RBASIC(v)->flags) { // liveness check
-            if (RB_TYPE_P(v, T_CLASS) ||
-                RB_TYPE_P(v, T_ICLASS)) {
-                if (RCLASS_CC_TBL(v)) {
-                    rb_cc_table_free(v);
+            if (imemo_type_p(v, imemo_callcache)) {
+                const struct rb_callcache *cc = (const struct rb_callcache *)v;
+                if (vm_cc_refinement_p(cc) && cc->klass) {
+                    vm_cc_invalidate(cc);
                 }
-                RCLASS_CC_TBL(v) = NULL;
             }
         }
+
         if (ptr) {
             asan_poison_object(v);
         }
@@ -328,10 +337,9 @@ invalidate_all_cc(void *vstart, void *vend, size_t stride, void *data)
 }
 
 void
-rb_clear_method_cache_all(void)
+rb_clear_all_refinement_method_cache(void)
 {
-    rb_objspace_each_objects(invalidate_all_cc, NULL);
-
+    rb_objspace_each_objects(invalidate_all_refinement_cc, NULL);
     rb_yjit_invalidate_all_method_lookup_assumptions();
 }
 
@@ -508,7 +516,7 @@ method_definition_addref(rb_method_definition_t *def, bool complemented)
     return def;
 }
 
-MJIT_FUNC_EXPORTED void
+void
 rb_method_definition_set(const rb_method_entry_t *me, rb_method_definition_t *def, void *opts)
 {
     rb_method_definition_release(me->def);
@@ -577,9 +585,7 @@ rb_method_definition_set(const rb_method_entry_t *me, rb_method_definition_t *de
             return;
           case VM_METHOD_TYPE_REFINED:
             {
-                const rb_method_refined_t *refined = (rb_method_refined_t *)opts;
-                RB_OBJ_WRITE(me, &def->body.refined.orig_me, refined->orig_me);
-                RB_OBJ_WRITE(me, &def->body.refined.owner, refined->owner);
+                RB_OBJ_WRITE(me, &def->body.refined.orig_me, (rb_method_entry_t *)opts);
                 return;
             }
           case VM_METHOD_TYPE_ALIAS:
@@ -615,7 +621,6 @@ method_definition_reset(const rb_method_entry_t *me)
         break;
       case VM_METHOD_TYPE_REFINED:
         RB_OBJ_WRITTEN(me, Qundef, def->body.refined.orig_me);
-        RB_OBJ_WRITTEN(me, Qundef, def->body.refined.owner);
         break;
       case VM_METHOD_TYPE_ALIAS:
         RB_OBJ_WRITTEN(me, Qundef, def->body.alias.original_me);
@@ -630,7 +635,7 @@ method_definition_reset(const rb_method_entry_t *me)
     }
 }
 
-MJIT_FUNC_EXPORTED rb_method_definition_t *
+rb_method_definition_t *
 rb_method_definition_create(rb_method_type_t type, ID mid)
 {
     rb_method_definition_t *def;
@@ -675,24 +680,40 @@ rb_method_entry_create(ID called_id, VALUE klass, rb_method_visibility_t visi, r
     return me;
 }
 
+// Return a cloned ME that's not invalidated (MEs are disposable for caching).
 const rb_method_entry_t *
 rb_method_entry_clone(const rb_method_entry_t *src_me)
 {
     rb_method_entry_t *me = rb_method_entry_alloc(src_me->called_id, src_me->owner, src_me->defined_class, src_me->def, METHOD_ENTRY_COMPLEMENTED(src_me));
 
     METHOD_ENTRY_FLAGS_COPY(me, src_me);
+
+    // Also clone inner ME in case of refinement ME
+    if (src_me->def &&
+            src_me->def->type == VM_METHOD_TYPE_REFINED &&
+            src_me->def->body.refined.orig_me) {
+        const rb_method_entry_t *orig_me = src_me->def->body.refined.orig_me;
+        VM_ASSERT(orig_me->def->type != VM_METHOD_TYPE_REFINED);
+
+        rb_method_entry_t *orig_clone = rb_method_entry_alloc(orig_me->called_id,
+                orig_me->owner, orig_me->defined_class, orig_me->def, METHOD_ENTRY_COMPLEMENTED(orig_me));
+        METHOD_ENTRY_FLAGS_COPY(orig_clone, orig_me);
+
+        // Clone definition, since writing a VALUE to a shared definition
+        // can create reference edges we can't run WBs for.
+        rb_method_definition_t *clone_def =
+            rb_method_definition_create(VM_METHOD_TYPE_REFINED, src_me->called_id);
+        rb_method_definition_set(me, clone_def, orig_clone);
+    }
     return me;
 }
 
-MJIT_FUNC_EXPORTED const rb_callable_method_entry_t *
+const rb_callable_method_entry_t *
 rb_method_entry_complement_defined_class(const rb_method_entry_t *src_me, ID called_id, VALUE defined_class)
 {
     rb_method_definition_t *def = src_me->def;
     rb_method_entry_t *me;
-    struct {
-        const struct rb_method_entry_struct *orig_me;
-        VALUE owner;
-    } refined = {0};
+    const rb_method_entry_t *refined_orig_me = NULL;
 
     if (!src_me->defined_class &&
         def->type == VM_METHOD_TYPE_REFINED &&
@@ -700,8 +721,7 @@ rb_method_entry_complement_defined_class(const rb_method_entry_t *src_me, ID cal
         const rb_method_entry_t *orig_me =
             rb_method_entry_clone(def->body.refined.orig_me);
         RB_OBJ_WRITE((VALUE)orig_me, &orig_me->defined_class, defined_class);
-        refined.orig_me = orig_me;
-        refined.owner = orig_me->owner;
+        refined_orig_me = orig_me;
         def = NULL;
     }
 
@@ -710,7 +730,7 @@ rb_method_entry_complement_defined_class(const rb_method_entry_t *src_me, ID cal
     METHOD_ENTRY_COMPLEMENTED_SET(me);
     if (!def) {
         def = rb_method_definition_create(VM_METHOD_TYPE_REFINED, called_id);
-        rb_method_definition_set(me, def, &refined);
+        rb_method_definition_set(me, def, (void *)refined_orig_me);
     }
 
     VM_ASSERT(RB_TYPE_P(me->owner, T_MODULE));
@@ -737,25 +757,19 @@ make_method_entry_refined(VALUE owner, rb_method_entry_t *me)
         return;
     }
     else {
-        struct {
-            struct rb_method_entry_struct *orig_me;
-            VALUE owner;
-        } refined;
         rb_method_definition_t *def;
 
         rb_vm_check_redefinition_opt_method(me, me->owner);
 
-        refined.orig_me =
+        struct rb_method_entry_struct *orig_me =
             rb_method_entry_alloc(me->called_id, me->owner,
-                                  me->defined_class ?
-                                  me->defined_class : owner,
+                                  me->defined_class ? me->defined_class : owner,
                                   me->def,
                                   true);
-        METHOD_ENTRY_FLAGS_COPY(refined.orig_me, me);
-        refined.owner = owner;
+        METHOD_ENTRY_FLAGS_COPY(orig_me, me);
 
         def = rb_method_definition_create(VM_METHOD_TYPE_REFINED, me->called_id);
-        rb_method_definition_set(me, def, (void *)&refined);
+        rb_method_definition_set(me, def, orig_me);
         METHOD_ENTRY_VISI_SET(me, METHOD_VISI_PUBLIC);
     }
 }
@@ -1006,7 +1020,7 @@ lookup_overloaded_cme(const rb_callable_method_entry_t *cme)
 }
 
 #if VM_CHECK_MODE > 0
-MJIT_FUNC_EXPORTED const rb_callable_method_entry_t *
+const rb_callable_method_entry_t *
 rb_vm_lookup_overloaded_cme(const rb_callable_method_entry_t *cme)
 {
     return lookup_overloaded_cme(cme);
@@ -1032,15 +1046,15 @@ get_overloaded_cme(const rb_callable_method_entry_t *cme)
     else {
         // create
         rb_method_definition_t *def = rb_method_definition_create(VM_METHOD_TYPE_ISEQ, cme->def->original_id);
-        def->body.iseq.cref = cme->def->body.iseq.cref;
-        def->body.iseq.iseqptr = ISEQ_BODY(cme->def->body.iseq.iseqptr)->mandatory_only_iseq;
-
         rb_method_entry_t *me = rb_method_entry_alloc(cme->called_id,
                                                       cme->owner,
                                                       cme->defined_class,
                                                       def,
                                                       false);
 
+        RB_OBJ_WRITE(me, &def->body.iseq.cref, cme->def->body.iseq.cref);
+        RB_OBJ_WRITE(me, &def->body.iseq.iseqptr, ISEQ_BODY(cme->def->body.iseq.iseqptr)->mandatory_only_iseq);
+
         ASSERT_vm_locking();
         st_insert(overloaded_cme_table(), (st_data_t)cme, (st_data_t)me);
 
@@ -1071,7 +1085,7 @@ check_overloaded_cme(const rb_callable_method_entry_t *cme, const struct rb_call
         VALUE recv_class = (klass);			\
         ID hook_id = (hook);				\
         if (FL_TEST((klass), FL_SINGLETON)) {		\
-            recv_class = rb_ivar_get((klass), attached);	\
+            recv_class = RCLASS_ATTACHED_OBJECT((klass));	\
             hook_id = singleton_##hook;			\
         }						\
         rb_funcallv(recv_class, hook_id, 1, &arg);	\
@@ -1095,7 +1109,7 @@ rb_add_method(VALUE klass, ID mid, rb_method_type_t type, void *opts, rb_method_
     }
 }
 
-MJIT_FUNC_EXPORTED void
+void
 rb_add_method_iseq(VALUE klass, ID mid, const rb_iseq_t *iseq, rb_cref_t *cref, rb_method_visibility_t visi)
 {
     struct { /* should be same fields with rb_method_iseq_struct */
@@ -1135,7 +1149,10 @@ void
 rb_define_alloc_func(VALUE klass, VALUE (*func)(VALUE))
 {
     Check_Type(klass, T_CLASS);
-    RCLASS_ALLOCATOR(klass) = func;
+    if (FL_TEST_RAW(klass, FL_SINGLETON)) {
+        rb_raise(rb_eTypeError, "can't define an allocator for a singleton class");
+    }
+    RCLASS_SET_ALLOCATOR(klass, func);
 }
 
 void
@@ -1207,7 +1224,7 @@ search_method_protect(VALUE klass, ID id, VALUE *defined_class_ptr)
     }
 }
 
-MJIT_FUNC_EXPORTED const rb_method_entry_t *
+const rb_method_entry_t *
 rb_method_entry(VALUE klass, ID id)
 {
     return search_method_protect(klass, id, NULL);
@@ -1301,7 +1318,6 @@ cache_callable_method_entry(VALUE klass, ID mid, const rb_callable_method_entry_
     VM_ASSERT(cme != NULL);
 
     struct rb_id_table *cc_tbl = RCLASS_CC_TBL(klass);
-    struct rb_class_cc_entries *ccs;
     VALUE ccs_data;
 
     if (!cc_tbl) {
@@ -1309,11 +1325,13 @@ cache_callable_method_entry(VALUE klass, ID mid, const rb_callable_method_entry_
     }
 
     if (rb_id_table_lookup(cc_tbl, mid, &ccs_data)) {
-        ccs = (struct rb_class_cc_entries *)ccs_data;
+#if VM_CHECK_MODE > 0
+        struct rb_class_cc_entries *ccs = (struct rb_class_cc_entries *)ccs_data;
         VM_ASSERT(ccs->cme == cme);
+#endif
     }
     else {
-        ccs = vm_ccs_create(klass, cc_tbl, mid, cme);
+        vm_ccs_create(klass, cc_tbl, mid, cme);
     }
 }
 
@@ -1385,7 +1403,7 @@ callable_method_entry(VALUE klass, ID mid, VALUE *defined_class_ptr)
     return !UNDEFINED_METHOD_ENTRY_P(cme) ? cme : NULL;
 }
 
-MJIT_FUNC_EXPORTED const rb_callable_method_entry_t *
+const rb_callable_method_entry_t *
 rb_callable_method_entry(VALUE klass, ID mid)
 {
     return callable_method_entry(klass, mid, NULL);
@@ -1416,14 +1434,14 @@ method_entry_resolve_refinement(VALUE klass, ID id, int with_refinement, VALUE *
     return me;
 }
 
-MJIT_FUNC_EXPORTED const rb_method_entry_t *
+const rb_method_entry_t *
 rb_method_entry_with_refinements(VALUE klass, ID id, VALUE *defined_class_ptr)
 {
     return method_entry_resolve_refinement(klass, id, TRUE, defined_class_ptr);
 }
 
 static const rb_callable_method_entry_t *
-callable_method_entry_refeinements0(VALUE klass, ID id, VALUE *defined_class_ptr, bool with_refinements,
+callable_method_entry_refinements0(VALUE klass, ID id, VALUE *defined_class_ptr, bool with_refinements,
                                     const rb_callable_method_entry_t *cme)
 {
     if (cme == NULL || LIKELY(cme->def->type != VM_METHOD_TYPE_REFINED)) {
@@ -1440,10 +1458,10 @@ static const rb_callable_method_entry_t *
 callable_method_entry_refinements(VALUE klass, ID id, VALUE *defined_class_ptr, bool with_refinements)
 {
     const rb_callable_method_entry_t *cme = callable_method_entry(klass, id, defined_class_ptr);
-    return callable_method_entry_refeinements0(klass, id, defined_class_ptr, with_refinements, cme);
+    return callable_method_entry_refinements0(klass, id, defined_class_ptr, with_refinements, cme);
 }
 
-MJIT_FUNC_EXPORTED const rb_callable_method_entry_t *
+const rb_callable_method_entry_t *
 rb_callable_method_entry_with_refinements(VALUE klass, ID id, VALUE *defined_class_ptr)
 {
     return callable_method_entry_refinements(klass, id, defined_class_ptr, true);
@@ -1461,7 +1479,7 @@ rb_method_entry_without_refinements(VALUE klass, ID id, VALUE *defined_class_ptr
     return method_entry_resolve_refinement(klass, id, FALSE, defined_class_ptr);
 }
 
-MJIT_FUNC_EXPORTED const rb_callable_method_entry_t *
+const rb_callable_method_entry_t *
 rb_callable_method_entry_without_refinements(VALUE klass, ID id, VALUE *defined_class_ptr)
 {
     VALUE defined_class, *dcp = defined_class_ptr ? defined_class_ptr : &defined_class;
@@ -1508,7 +1526,6 @@ rb_resolve_refined_method(VALUE refinements, const rb_method_entry_t *me)
     return resolve_refined_method(refinements, me, NULL);
 }
 
-MJIT_FUNC_EXPORTED
 const rb_callable_method_entry_t *
 rb_resolve_refined_method_callable(VALUE refinements, const rb_callable_method_entry_t *me)
 {
@@ -2057,7 +2074,7 @@ original_method_definition(const rb_method_definition_t *def)
     return def;
 }
 
-MJIT_FUNC_EXPORTED int
+int
 rb_method_definition_eq(const rb_method_definition_t *d1, const rb_method_definition_t *d2)
 {
     d1 = original_method_definition(d1);
@@ -2092,7 +2109,7 @@ rb_method_definition_eq(const rb_method_definition_t *d1, const rb_method_defini
       case VM_METHOD_TYPE_ALIAS:
         break;
     }
-    rb_bug("rb_method_definition_eq: unsupported type: %d\n", d1->type);
+    rb_bug("rb_method_definition_eq: unsupported type: %d", d1->type);
 }
 
 static st_index_t
@@ -2105,7 +2122,7 @@ rb_hash_method_definition(st_index_t hash, const rb_method_definition_t *def)
 
     switch (def->type) {
       case VM_METHOD_TYPE_ISEQ:
-        return rb_hash_uint(hash, (st_index_t)def->body.iseq.iseqptr->body);
+        return rb_hash_uint(hash, (st_index_t)def->body.iseq.iseqptr);
       case VM_METHOD_TYPE_CFUNC:
         hash = rb_hash_uint(hash, (st_index_t)def->body.cfunc.func);
         return rb_hash_uint(hash, def->body.cfunc.argc);
@@ -2127,7 +2144,7 @@ rb_hash_method_definition(st_index_t hash, const rb_method_definition_t *def)
       case VM_METHOD_TYPE_ALIAS:
         break; /* unreachable */
     }
-    rb_bug("rb_hash_method_definition: unsupported method type (%d)\n", def->type);
+    rb_bug("rb_hash_method_definition: unsupported method type (%d)", def->type);
 }
 
 st_index_t
@@ -2552,7 +2569,7 @@ rb_mod_private_method(int argc, VALUE *argv, VALUE obj)
 static VALUE
 top_public(int argc, VALUE *argv, VALUE _)
 {
-    return rb_mod_public(argc, argv, rb_top_main_class("public"));
+    return rb_mod_public(argc, argv, rb_cObject);
 }
 
 /*
@@ -2572,7 +2589,7 @@ top_public(int argc, VALUE *argv, VALUE _)
 static VALUE
 top_private(int argc, VALUE *argv, VALUE _)
 {
-    return rb_mod_private(argc, argv, rb_top_main_class("private"));
+    return rb_mod_private(argc, argv, rb_cObject);
 }
 
 /*
@@ -2585,7 +2602,7 @@ top_private(int argc, VALUE *argv, VALUE _)
 static VALUE
 top_ruby2_keywords(int argc, VALUE *argv, VALUE module)
 {
-    return rb_mod_ruby2_keywords(argc, argv, rb_top_main_class("ruby2_keywords"));
+    return rb_mod_ruby2_keywords(argc, argv, rb_cObject);
 }
 
 /*
@@ -2866,12 +2883,6 @@ obj_respond_to_missing(VALUE obj, VALUE mid, VALUE priv)
     return Qfalse;
 }
 
-void
-Init_Method(void)
-{
-    //
-}
-
 void
 Init_eval_method(void)
 {
